#!/usr/bin/perl
# tweaksvg.pl          Norbert Haider, University of Vienna
#
# Script reads an SVG file (created by mol2svg v0.3) and adjusts
# the width, height and viewbox dimensions according to the corresponding
# comment lines which are present at the end of the original SVG file.
#

$scale       = 0.75;
$keep_xydata = 1;



if ($#ARGV < 0) {
  print "Usage: tweaksvg.pl <inputfile>\n";
  exit;
}

$infile = $ARGV[0];

open (SVG, "<$infile") || die ("cannot open svg file $filename!");

$molsvg = "";

while ($line = <SVG>) {
  $line =~ s/\r//g;
  $molsvg .= $line;
}

close(SVG);

$newsvg = tweak_svg($molsvg);


sub tweak_svg {
  my $testsvg = shift;
  my @svgline = split(/\n/, $testsvg);
  my $xmaxval = "";
  my $yminval = "";
  my $ytrval = "";
  my $scaling = $scale;
  for ($i = 0; $i <= $#svgline; $i++) {
    my $testline = $svgline[$i];
    chomp($testline);
    if ((index($testline,"\<!-- max_X:") >= 0) && (index($testline,"--\>") >= 0)) {
      my @xline = split(/:/, $testline);
      $xmaxval = $xline[1];
      $xmaxval =~ s/--\>//g;
      chomp($xmaxval);
      $xmaxval =~ s/\ +//g;
      if ($keep_xydata == 0) { $svgline[$i] = ""; }
    }
    if ((index($testline,"\<!-- max_Y:") >= 0) && (index($testline,"--\>") >= 0)) {
      my @yline = split(/:/, $testline);
      $ymaxval = $yline[1];
      $ymaxval =~ s/--\>//g;
      chomp($ymaxval);
      if ($keep_xydata == 0) { $svgline[$i] = ""; }
    }
    if ((index($testline,"\<!-- min_Y:") >= 0) && (index($testline,"--\>") >= 0)) {
      my @yline = split(/:/, $testline);
      $yminval = $yline[1];
      $yminval =~ s/--\>//g;
      chomp($yminval);
      if ($keep_xydata == 0) { $svgline[$i] = ""; }
    }
    if ((index($testline,"\<!-- yshift:") >= 0) && (index($testline,"--\>") >= 0)) {
      my @ytrline = split(/:/, $testline);
      $ytrval = $ytrline[1];
      $ytrval =~ s/--\>//g;
      chomp($ytrval);
      if ($keep_xydata == 0) { $svgline[$i] = ""; }
    }
    if ((index($testline,"\<!-- found XY values for adjusting") >= 0) && (index($testline,"--\>") >= 0)) {
      if ($keep_xydata == 0) { $svgline[$i] = ""; }
    }
  }  # for
  if ((length(xmaxval) > 0) && (length(ymaxval) > 0) && (length(yminval) > 0) && (length(ytrval) > 0)) {
    my $ymaxtotal   = $ymaxval + $ytrval;
    my $ymintotal   = $yminval + $ytrval;
    my $ydiff       = $ymaxtotal - $ymintotal;
    my $ydiffscaled = $ydiff * $scaling;
    my $xmaxscaled  = $xmaxval * $scaling;
    my $ymaxscaled  = $ymaxtotal * $scaling;
    my $yminscaled  = $ymintotal * $scaling;
    $svgline[1] = "<svg width=\"$xmaxscaled\" height=\"$ydiffscaled\" viewbox=\"0 $ymintotal $xmaxval $ydiff\" xmlns=\"http://www.w3.org/2000/svg\">";
  }
  my $twsvg = join("\n",@svgline);
  $twsvg =~ s/\n\n\n//g;
  return $twsvg;
}

sub ltrim() {
  $subline1 = shift;
  $subline1 =~ s/^\ +//g;
  return $subline1;
}
