#!/usr/bin/perl
#
# upgrade5to6.pl      Norbert Haider, University of Vienna, 2014-2018
#                     norbert.haider@univie.ac.at
#
# This script is part of the MolDB6 package. Last change: 2018-01-05
#
# Example script which upgrades an existing MolDB5R installation
# to MolDB6. It expects already a moldb6.conf configuration file.
# The script adds the column "flags" to the moldb_meta table and
# it adds the new "auto_mol_" fileds to all moldata tables.

$level = 2;  # 0 = dryrun, 1 = only moldb_meta, 2 = everything

use DBI();

$configfile = "../moldb6.conf";

$return     = do $configfile;
if (!defined $return) {
  die("ERROR: cannot read configuration file $configfile!\n");
}	

$enable_inchi = "y";

$user     = $rw_user;    # from configuration file
$password = $rw_password;

$dbh = DBI->connect("DBI:mysql:database=$database;host=$hostname",
                    $user, $password,
                    {'RaiseError' => 1});
$qstr = "SET SESSION sql_mode = '';";
$dbh->do($qstr);

# read moldb_meta table

$found_flags = 0;
$sth0 = $dbh->prepare("SHOW FULL COLUMNS FROM $metatable");
$sth0->execute();
while ($ref0 = $sth0->fetchrow_hashref()) {
  $field     = $ref0->{'Field'};
  $type    = $ref0->{'Type'};
  print "$field : $type \n";
  if ($field eq "flags") { $found_flags = 1; }
}
$sth0->finish;

if ($found_flags == 1) {
  print "The moldb_meta table is already in MolDB6 format.\n";
} else {
  $updstr = "ALTER TABLE `$metatable` ADD `flags` INT(11) UNSIGNED NOT NULL DEFAULT '0' AFTER `trustedIP`";
  print "$updstr\n";
  if ($level > 0) {
    $dbh->do($updstr);	  
  }
}


$ndb = 0;
$changed = 0;
$sth0 = $dbh->prepare("SELECT db_id FROM $metatable WHERE (type = 1) ORDER BY db_id");
$sth0->execute();
while ($ref0 = $sth0->fetchrow_hashref()) {
  $db_id     = $ref0->{'db_id'};
  $ndb++;
  print "   examining data collection $db_id\n";
  $moldatatable = $prefix . "db" . $db_id . "_" . $moldatasuffix;
  $qstr = "SHOW FULL COLUMNS FROM $moldatatable";
  #print "     $qstr\n";
  $sth1 = $dbh->prepare($qstr);
  $sth1->execute();
  $found_auto_mol_formula = 0;
  $found_auto_mol_fw = 0;
  $found_auto_mol_inchikey = 0;
  while ($ref1 = $sth1->fetchrow_hashref()) {
    $field =  $ref1->{'Field'};
    $type =  $ref1->{'Type'};
    if ($field eq "auto_mol_formula") { $found_auto_mol_formula = 1; }
    if ($field eq "auto_mol_fw") { $found_auto_mol_fw = 1; }
    if ($field eq "auto_mol_inchikey") { $found_auto_mol_inchikey = 1; }
    #print "              $field     $Type \n";
  }
  $sth1->finish;
  $addstr = "";
  if ($found_auto_mol_formula == 0) {
    $addstr = "ADD `auto_mol_formula` VARCHAR(250) NOT NULL COMMENT '>>>>Formula<3<auto_mol_formula<0<<' AFTER `mol_id`";
  }
  if ($found_auto_mol_fw == 0) {
    if (length($addstr) > 0) { $addstr .= ", "; }
    $addstr .= "ADD `auto_mol_fw` DOUBLE NOT NULL DEFAULT '0.0' COMMENT '>>>>MW<1<auto_mol_fw<0<<' AFTER `auto_mol_formula`";
  }

  if (($found_auto_mol_inchikey == 0) && ($enable_inchi eq "y")) {
    if (length($addstr) > 0) { $addstr .= ", "; }
    $addstr .= "ADD `auto_mol_inchikey` VARCHAR(250) NOT NULL COMMENT '>>>>InChIKey<1<inchikey<1<<' AFTER `auto_mol_fw`";
  }
  if (length($addstr) > 0) {
    $addstr = "ALTER TABLE $moldatatable " . $addstr;
    print " $addstr \n";
  }
  if ((length($addstr) > 0) && ($level > 1)) {
    $dbh->do($addstr);
    $changed++;
  }
}

$sth0->finish;

if ($changed > 0) {
  print "Upgraded data collections: $changed \n";
  print "Do not forget to run the script mkautomol.pl to fill the auto_mol_ columns with values.\n";
}

exit;

if ($ndb < 1) {
  print "no such structure data collection ($db_id)\n";
  exit;
}

if ($setusemem eq $usemem) {
  print "setting of 'usemem' for data collection $dbnum is already '" . $usemem . "'\n"	;
  exit;
} else {
  $updstr = "UPDATE $metatable SET usemem = '" . $setusemem . "' WHERE db_id = $dbnum";
  $dbh->do($updstr);	
  $updstr = "UPDATE $metatable SET memstatus = '0' WHERE db_id = $dbnum";
  $dbh->do($updstr);	
}

if ($setusemem eq "T") {
  print "OK. Do not forget to run 'cp2mem.pl' in order to synchronize memory tables!\n";
  if ($access > 1) {
    print "Attention: using memory-based tables is NOT recommended for read/write\n";
    print "data collections!\n";
  }
} else {
  print "OK.\n";
}

$dbh->disconnect();

