<?php
// popupview.php   Norbert Haider, University of Vienna, 2018
// part of MolDB6  last change: 2018-01-08

/**
 * @file popupview.php
 * @author Norbert Haider
 * 
 * This script is invoced when a user clicks on the "view in pop-up window" 
 * hyperlink of an item (molecule or reaction) on a details page. It displays 
 * either a spectrum (in JCAMP-DX format) or a (3D) molfile by means of
 * the JavaScript applet JSmol nd its companion, JSV. These applets must be
 * installed on the server in order to make use of this feature. The format of
 * the corresponding data field must be set to 5 (JSV mode) or 6 (JSmol mode).
 */


// some default values; will be overridden by the config files)
$sitename       = "MolDB6 demo";
$cssfilename    = "moldb.css";
$jsmoldir       = "../jsmol";  // path to the jsmol directory (without trailing slash)
$jspopup_width  = 1024;
$jspopup_height = 700;

$myname = $_SERVER['PHP_SELF'];
@include("moldb6conf.php");
@include("moldb6uiconf.php");

require_once("functions.php");

if (config_quickcheck() > 0) { die(); }
set_charset($charset);

if (!isset($sitename) || ($sitename == "")) {
  $sitename = "MolDB6 demo";
}
@$datafile = $_REQUEST['data'];
@$datatype = $_REQUEST['type'];
$validtype = 0;

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=<?php echo "$html_charset"; ?>">
<meta name="author" content="Norbert Haider, University of Vienna">
<title><?php echo "$sitename (viewer window)"; ?></title>

<?php
if ($datatype === "jsv") {
  $validtype = 1;
  $jsv_width = $jspopup_width - 24;
  $jsv_height = $jspopup_height - 60;

?>
<script type="text/javascript" src="<?php echo "$jsmoldir";?>/JSmol.min.js"></script>
<script type="text/javascript" src="<?php echo "$jsmoldir";?>/js/JSmolMenu.js"></script>  
<script type="text/javascript" src="<?php echo "$jsmoldir";?>/js/JSmolJSV.js"></script>

<script type="text/javascript">

jsvApplet = "jsvApplet0";

var SInfo;
  ;(function() {
  use="HTML5";

  SInfo = {
	height: <?php echo "$jsv_height"; ?>,
	width: <?php echo "$jsv_width"; ?>,
	debug: false,
	color: "0xC0C0C0",
	use: "HTML5",
        j2sPath: "<?php echo "$jsmoldir";?>/j2s",
//        jarPath: "<?php echo "$jsmoldir";?>/java",
	disableJ2SLoadMonitor: true,
	disableInitialConsole: true,
        allowjavascript: true,
	script: "load '<?php echo "$datafile"; ?>';"
  }
})();

</script>

<?php 
}  // if dattype == jsv

if ($datatype === "jsmol") {
  $validtype = 1;
  $jsmol_width = $jspopup_width - 224;
  $jsmol_height = $jspopup_height - 60;
?>
<script type="text/javascript" src="<?php echo "$jsmoldir";?>/JSmol.min.js"></script>
<script type="text/javascript" src="<?php echo "$jsmoldir";?>/js/JSmolMenu.js"></script>  

<script type="text/javascript">

jmolApplet = "jmolApplet0";

var MInfo;
  ;(function() {
  use="HTML5";

  MInfo = {
	height: <?php echo "$jsmol_height"; ?>,
	width: <?php echo "$jsmol_width"; ?>,
	debug: false,
	color: "0x000000",
	use: "HTML5",
        j2sPath: "<?php echo "$jsmoldir";?>/j2s",
//        jarPath: "<?php echo "$jsmoldir";?>/java",
	disableJ2SLoadMonitor: true,
	disableInitialConsole: true,
        allowjavascript: true,
	script: "load '<?php echo "$datafile"; ?>';"
  }
})();

</script>

<?php 
}  // if datatype == jsmol

insert_style("$cssfilename"); 
if ($validtype == 0) {
  echo "invalid data type - aborting...<br>\n</body>\n</html>\n";
  die();
}

?>
</head>
<body>

<?php
if ($datatype === "jsv") {
?>
<script>
  jsvApplet = Jmol.getJSVApplet("jsvApplet0",SInfo);
</script><hr>

<?php
}

if ($datatype === "jsmol") {
?>
<table>
<tr>
<td>
		
<script>
  jmolApplet = Jmol.getApplet("jmolApplet0", MInfo);
</script>
</td>
<td>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spacefill off; wireframe on; ')">Wireframe</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spacefill 20%; wireframe 0.15; ')">Ball & stick</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spacefill on ')">Spacefill</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spin on ')">Rotation on</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spin off ')">Rotation off</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'zoom 50; zoom on ')">Zoom 50%</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'zoom 100; zoom on ')">Zoom 100%</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'zoom 200; zoom on ')">Zoom 200%</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'dots off; set solvent off; dots on ')">van der Waals surface</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'dots off; set solvent on; dots on ')">Conolly surface</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'dots off ')">No surface</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spacefill off; wireframe 0.1; color atoms grey; select nitrogen; color green; spacefill 50% ')">Select N-atoms</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'spacefill off; wireframe 0.1; color atoms grey; select oxygen; color purple; spacefill 50% ')">Select O-atoms</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'select *; spacefill off; wireframe on; color atoms CPK; ')">Cancel selection</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'label %e; set labeloffset 0 0; font label 12; color label black')">Labels: by element</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'label %i; set labeloffset 0 0; font label 12; color label black')">Labels: by atom ID</a><br>
<a class="menu" href="javascript:Jmol.script(jmolApplet0,'select *; label off')">Labels off</a><br>
</td>
</tr>
</table>

<?php
}
?>
<form>
<p align="center">
<input type="button" value="Close" onClick="window.close()"> </p>
</form>

</body>
</html>
